#!/usr/bin/perl -w

die "Change this script to use a version of Perl greater than or equal to 5."
    if $PERL_VERSION < 5;

use strict;
use English;
use Getopt::Long;

use DBI;
use ba_mysql_shared;

use constant false => 0;
use constant true => 1;

my $options_new_db;
my $options_renew_db;

######################################################################

&parse_options;

&db_connect(undef, "127.0.0.1", "root", $db_root_pw);

if($options_new_db) {
    &install_new_db;
}

if($options_renew_db) {
    &renew_db;
}

&db_disconnect;

exit 0;

sub install_new_db {
    &db_new_db($options_new_db);
}

sub renew_db {
    &db_renew_db($options_renew_db);
}

######################################################################

sub parse_options
{
    my %opt_hash;
    my $num_actions;

    $Getopt::Long::autoabbrev = false;
    $Getopt::Long::order = $REQUIRE_ORDER;
    $Getopt::Long::ignorecase = false;

    $num_actions = 0;
    $options_new_db = undef;
    $options_renew_db = undef;

    if(GetOptions(\%opt_hash,
		  "new-db=s",
		  "renew-db=s") == false) {
	die "bad option(s): error in GetOptions()";
    }

    if(defined($opt_hash{"new-db"})) {
	$options_new_db = $opt_hash{"new-db"};
	$num_actions++;
    }

    if(defined($opt_hash{"renew-db"})) {
	$options_renew_db = $opt_hash{"renew-db"};
	$num_actions++;
    }

    die "too many actions" if($num_actions > 1);
    die "not enough actions" if($num_actions == 0);
}
