#include <ba_config.h>
#include <ba_sim.h>

extern int main(int argc, char **argv);

extern int
main(int argc,
     char **argv)
{
    ba_status status, error_status;
    GHookList *shutdown_list;

    error_status = BA_STATUS_SUCCESS;

    status = ba_glib_init();
    BA_ASSERT(status == BA_STATUS_SUCCESS);
    shutdown_list = ba_glib_hook_new();

    status = ba_assert_init();
    BA_ASSERT(status == BA_STATUS_SUCCESS);
    ba_glib_hook_add(shutdown_list, ba_assert_destroy_from_hook);

    // this has to come real early...
    status = ba_mem_init();
    BA_ASSERT(status == BA_STATUS_SUCCESS);
    ba_glib_hook_add(shutdown_list, ba_mem_destroy_from_hook);

    // has to be before options...
    status = ba_rv_init();
    BA_ASSERT(status == BA_STATUS_SUCCESS);
    ba_glib_hook_add(shutdown_list, ba_rv_destroy_from_hook);

    status = ba_options_init();
    BA_ASSERT(status == BA_STATUS_SUCCESS);
    ba_glib_hook_add(shutdown_list, ba_options_destroy_from_hook);

    status = ba_options_get_args(argc, argv);
    BA_ASSERT(status == BA_STATUS_SUCCESS);

    // must init print before parsing the args, since parsing the args
    // uses the print routines.
    status = ba_print_init();
    if(status == BA_STATUS_PRINT_BAD_DATA_OUTPUT_FILENAME ||
       status == BA_STATUS_PRINT_BAD_TRACE_OUTPUT_FILENAME ||
       status == BA_STATUS_PRINT_BAD_TRACE_INPUT_FILENAME) {
	error_status = status;
	goto end;
    } else {
	BA_ASSERT(status == BA_STATUS_SUCCESS);
    }
    ba_glib_hook_add(shutdown_list, ba_print_destroy_from_hook);

    status = ba_options_parse_args();
    if(status == BA_STATUS_OPTIONS_BAD ||
       status == BA_STATUS_OPTIONS_EXIT) {
	error_status = status;
	goto end;
    } else {
	BA_ASSERT(status == BA_STATUS_SUCCESS);
    }

    status = ba_report_init();
    BA_ASSERT(status == BA_STATUS_SUCCESS);
    ba_glib_hook_add(shutdown_list, ba_report_destroy_from_hook);

    status = ba_trace_init();
    BA_ASSERT(status == BA_STATUS_SUCCESS);
    ba_glib_hook_add(shutdown_list, ba_trace_destroy_from_hook);

    status = ba_behavior_init();
    BA_ASSERT(status == BA_STATUS_SUCCESS);
    ba_glib_hook_add(shutdown_list, ba_behavior_destroy_from_hook);

    status = ba_events_init();
    BA_ASSERT(status == BA_STATUS_SUCCESS);
    ba_glib_hook_add(shutdown_list, ba_events_destroy_from_hook);

    status = ba_users_init();
    BA_ASSERT(status == BA_STATUS_SUCCESS);
    ba_glib_hook_add(shutdown_list, ba_users_destroy_from_hook);

    status = ba_user_init();
    BA_ASSERT(status == BA_STATUS_SUCCESS);
    ba_glib_hook_add(shutdown_list, ba_user_destroy_from_hook);

    status = ba_sched_init();
    BA_ASSERT(status == BA_STATUS_SUCCESS);
    ba_glib_hook_add(shutdown_list, ba_sched_destroy_from_hook);

    status = ba_resource_init();
    BA_ASSERT(status == BA_STATUS_SUCCESS);
    ba_glib_hook_add(shutdown_list, ba_resource_destroy_from_hook);

    //
    //ba_tests_running_mean();
    //ba_tests_dist();
    //ba_tests_time_periods();
    //

    ba_users_start();
    ba_events_simulate();
    ba_report_output();

    //g_mem_profile();

 end:
    g_hook_list_invoke(shutdown_list, FALSE);
    ba_glib_hook_delete(shutdown_list);

    status = ba_glib_destroy();
    BA_ASSERT(status == BA_STATUS_SUCCESS);

    if(error_status) {
	status = error_status;
    }
    
    return status;
}
