package ba_stat_nums;
use strict;

use Math::BigFloat;
use Math::BigInt lib => 'Pari';

use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

use Exporter;
$VERSION = 1.00;
@ISA = qw(Exporter);

@EXPORT = qw( &stat_nums_compute &stat_nums_print );
@EXPORT_OK = qw( );
%EXPORT_TAGS = ( );

use vars qw ( );

######################################################################

use constant true => 1;
use constant false => 0;

sub stat_nums_compute
{
    my $all_values = shift;
    my ($count, $total, $min, $max, $sample_mean,
	$sample_variance, $standard_deviation, $standard_error, $conf_delta_90,
	$conf_delta_95,	$conf_delta_99);
    my ($total_squared_deviance, $deviance);
    my %stats;
    my ($geo_prod, $geo_mean, $geo_count);

    $count = @$all_values;

    die "count is 0 in stat_nums_compute" if($count == 0);

    #    print "count: $count.\n";
    #    print "vals: @$all_values.\n";

    $min = $max = $$all_values[0];
    $total = 0;
    foreach my $value (@$all_values) {
	$total += $value;
	$max = $value if($value > $max);
	$min = $value if($value < $min);
    }

    $sample_mean = $total / $count;

    $total_squared_deviance = 0;
    foreach my $value (@$all_values) {
	$deviance = $value - $sample_mean;
	$total_squared_deviance += $deviance * $deviance;
    }

    if($count > 1) {
	$sample_variance = $total_squared_deviance / ($count - 1);
    } else {
	$sample_variance = 0;
    }
    
    $standard_deviation = sqrt($sample_variance);
    $standard_error = $standard_deviation / sqrt($count);

    $conf_delta_90 = 1.65 * $standard_error;
    $conf_delta_95 = 1.96 * $standard_error;
    $conf_delta_99 = 2.48 * $standard_error;

# # big nums
#     $geo_prod = Math::BigFloat->bone();
# # regular doubles:
# #    $geo_prod = 1.0;

#     $geo_count = 0;
#     foreach my $value (@$all_values) {
# 	if($value <= 0.0) {
# 	    next;
# 	}
# 	$geo_count++;
# 	$geo_prod *= $value;
# #	print STDERR "$geo_count\n";
#     }
#     if($geo_count != 0.0) {
# 	my $geo_inv_count;

# 	$geo_inv_count = 1.0 / $geo_count;

# #	print STDERR "b4 mean\n";
# 	$geo_mean = $geo_prod->bpow($geo_inv_count);
# # a little slower for big nums...  works for both big nums and regular doubles:
# #	$geo_mean = $geo_prod ** $geo_inv_count;
# #	print STDERR "after mean\n";
#     } else{
# 	$geo_mean = 0.0;
#     }

###

# big nums
#    $geo_prod = Math::BigFloat->bone();
# regular doubles:
    $geo_prod = 0.0;

    $geo_count = 0;
    foreach my $value (@$all_values) {
	if($value <= 0.0) {
	    next;
	}
	$geo_count++;
	$geo_prod += log $value;
    }
    if($geo_count != 0.0) {
	my $geo_inv_count;

	$geo_inv_count = 1.0 / $geo_count;

	$geo_mean = exp($geo_inv_count * $geo_prod);
    } else{
	$geo_mean = 0.0;
    }

###

    $stats{count} = $count;
    $stats{total} = $total;
    $stats{min} = $min;
    $stats{max} = $max;
    $stats{sample_mean} = $sample_mean;
    $stats{sample_variance} = $sample_variance;
    $stats{standard_deviation} = $standard_deviation;
    $stats{standard_error} = $standard_error;
    $stats{conf_delta_90} = $conf_delta_90;
    $stats{conf_delta_95} = $conf_delta_95;
    $stats{conf_delta_99} = $conf_delta_99;
    $stats{geo_mean} = $geo_mean;

#    print "foo: $stats{geo_mean}\n";

    return \%stats;
}

sub dp_pow
{
# uh, maybe ** is a normal perl operator!!  don't need dp_pow...
    my ($a, $b) = @_;
    my ($ln_a, $to_e, $answer);

    if($a <= 0.0) {
	return $a;
    }

    $ln_a = log $a;
    $to_e = $b * $ln_a;
    $answer = exp $to_e;

#    print "a: $a, b: $b, answer: $answer\n";

    return $answer;
}

sub stat_nums_print
{
    my $all_values = shift;
    my $stats;

    $stats = &stat_nums_compute($all_values);

#    print "count\ttotal\tmin\tmax\tmean\tvar\tstddev\tstderr\t" .
#	"conf90\tconf95\tconf99\n";
    
#    printf("%d\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\t%.3f\n",
#	   $count, $total, $min, $max, $sample_mean, $sample_variance,
#	   $standard_deviation, $standard_error, $conf_delta_90,
#	   $conf_delta_95, $conf_delta_99);

    print "mean\ttotal\tmin - max\tstddev\tconf_95\tgeo_mean\n";
    printf("%.6f\t%.6f\t%.6f-%.6f\t%.6f\t%.6f\t%.6f\n",
	   $stats->{sample_mean}, $stats->{total}, $stats->{min},
	   $stats->{max}, $stats->{standard_deviation},
	   $stats->{conf_delta_95}, $stats->{geo_mean});
}

######################################################################

1;
