#include <ba_config.h>
#include <string.h>
#include <ba_sim.h>

extern ba_bool
ba_string_scan_n_reals(char *in,
		       uint32_t n,
		       ba_real *out)
{
    BA_ASSERT(in && out);
    BA_ASSERT(n > 0);
    uint32_t i;
    ba_bool ok;
    char *endptr;

    for(i = 0; i < n; i++) {
	if(i == n - 1) {

	    // special case, last number
	    ok = ba_string_scan_real(in, &out[i]);
	    if(ok == BA_FALSE) {
		return BA_FALSE;
	    }

	} else {

	    ok = ba_string_scan_up_to(in, &endptr, ',');
	    if(ok == BA_FALSE) {
		return BA_FALSE;
	    }
	    ok = ba_string_scan_real(in, &out[i]);
	    if(ok == BA_FALSE) {
		return BA_FALSE;
	    }
	    in = endptr + 1;

	}
    }

    return BA_TRUE;
}

extern ba_bool
ba_string_scan_real(char *in,
		    ba_real *out)
{
    BA_ASSERT(in);
    BA_ASSERT(out);
    char *endptr;
    
    if(*in == '\000') {
	return BA_FALSE;
    }
    
    *out = strtod(in, &endptr);
    if(*endptr != '\000') {
	return BA_FALSE;
    }

    return BA_TRUE;
}

extern ba_bool
ba_string_scan_up_to(char *in,
		     char **out,
		     char delim)
{
    BA_ASSERT(in);
    BA_ASSERT(out);

    while(*in != '\000' && *in != delim) {
	in++;
    }
    if(*in == '\000') {
	return BA_FALSE;
    }

    *in = '\000';
    *out = in;
    return BA_TRUE;
}

extern char *
ba_string_append(char *head_str,
		 char *tail_str)
{
    BA_ASSERT(head_str);
    BA_ASSERT(tail_str);
    char *str;
    uint32_t len;

    len = strlen(head_str) + strlen(tail_str);

    str = ba_string_new_len(head_str, len);
    ba_string_delete(head_str);
    strcat(str, tail_str);

    return str;
}

extern char *
ba_string_new(char *str_src)
{
    BA_ASSERT(str_src);
    char *str_dst;

    str_dst = ba_mem_malloc(strlen(str_src) + 1);
    strcpy(str_dst, str_src);

    return str_dst;
}

extern char *
ba_string_new_len(char *str_src,
		  uint32_t len)
{
    char *str_dst;

    str_dst = ba_mem_malloc(len + 1);
    if(str_src) {
	BA_ASSERT(strlen(str_src) <= len);
	strcpy(str_dst, str_src);
    }

    return str_dst;
}

extern void
ba_string_delete(char *s)
{
    BA_ASSERT(s);

    ba_mem_free(s);
}
