#include <ba_config.h>
#include <ba_sim.h>

extern void
ba_tests_running_mean(void)
{
    ba_stats_running_mean_t *ra_stats;

    ra_stats = ba_stats_running_mean_new();

    ba_stats_running_mean_insert(ra_stats, 2, 1);
    ba_stats_running_mean_insert(ra_stats, 3, 2);
    ba_stats_running_mean_insert(ra_stats, 5, 3);

    ba_stats_running_mean_print(ra_stats);

    ba_print_info("foo: " BA_REAL_FMT "\n",
		  ba_stats_running_mean_get(ra_stats));

    ba_stats_running_mean_delete(ra_stats);
}

extern void
ba_tests_dist(void)
{
    uint32_t iter;
    uint32_t i;
    ba_real rv;
    ba_rv_dist_t *dist;
    ba_stats_t *stats;

    stats = ba_stats_new();

    iter = 1000000;

    //    dist = ba_rv_dist_new("pareto", -1.0, -1.0, 1.5, 0.0);
    dist = ba_rv_dist_new("pareto", 10.0, 1000.0, 1.5, 0.0);
    //    dist = ba_rv_dist_new("exponential", 0.0, 0.0, 10.0, 0.0);
    //    dist = ba_rv_dist_new("bernoulli", 0.0, 0.0, 0.0, 0.1);
    BA_ASSERT(dist);
    ba_rv_dist_print(dist);

    ba_print_info("(* iterations: %d *)\n", iter);

    for(i = 0; i < iter; i++) {
	rv = ba_rv_dist(dist);
	ba_print_info(BA_REAL_FMT "\n", rv);

	ba_stats_insert(stats, rv);
    }

    ba_print_data("dist_stats: ");
    ba_stats_print_nice_to(stats, ba_print_data);
    ba_print_data("\n");

    ba_rv_dist_delete(dist);

    ba_stats_delete(stats);
}

extern void
ba_tests_time_periods(void)
{
    {
	ba_time_periods_t *periods_a, *periods_b, *result;
	
	ba_print_info("INTERSECTION TEST.......\n");

	periods_a = ba_time_periods_new();
	periods_b = ba_time_periods_new();
	result = ba_time_periods_new();

	ba_print_info("a (setup):\n");
	ba_time_periods_append_start_end(periods_a, 0.0, 1.0);
	
	ba_time_periods_append_start_end(periods_a, 10.0, 11.0);
	ba_time_periods_append_start_end(periods_a, 20.0, 21.0);
	ba_time_periods_append_start_end(periods_a, 30.0, 31.0);
	ba_time_periods_append_start_end(periods_a, 40.0, 41.0);

	ba_time_periods_append_start_end(periods_a, 50.5, 51.0);
	ba_time_periods_append_start_end(periods_a, 60.5, 61.5);
	ba_time_periods_append_start_end(periods_a, 71.0, 72.0);
	ba_time_periods_append_start_end(periods_a, 82.0, 83.0);
	
	ba_time_periods_append_start_end(periods_a, 90.0, 91.0);
	ba_time_periods_append_start_end(periods_a, 95.1, 95.9);

	ba_time_periods_print(periods_a);
	
	ba_print_info("b (setup):\n");
	ba_time_periods_append_start_end(periods_b, 0.0, 1.0);

	ba_time_periods_append_start_end(periods_b, 10.5, 11.0);
	ba_time_periods_append_start_end(periods_b, 20.5, 21.5);
	ba_time_periods_append_start_end(periods_b, 31.0, 32.0);
	ba_time_periods_append_start_end(periods_b, 42.0, 43.0);

	ba_time_periods_append_start_end(periods_b, 50.0, 51.0);
	ba_time_periods_append_start_end(periods_b, 60.0, 61.0);
	ba_time_periods_append_start_end(periods_b, 70.0, 71.0);
	ba_time_periods_append_start_end(periods_b, 80.0, 81.0);
	
	ba_time_periods_append_start_end(periods_b, 90.1, 90.9);
	ba_time_periods_append_start_end(periods_b, 95.0, 96.0);

	ba_time_periods_print(periods_b);
	
	ba_print_info("a ^ b:\n");
	ba_time_periods_intersection(periods_a, periods_b, result);
	ba_time_periods_print(result);
	
	ba_time_periods_delete(periods_a);
	ba_time_periods_delete(periods_b);
	ba_time_periods_delete(result);
    }

    {
	ba_time_periods_t *periods_a, *periods_b, *result;

	ba_print_info("DIFF TESTS...........................\n");

	periods_a = ba_time_periods_new();
	periods_b = ba_time_periods_new();
	result = ba_time_periods_new();

	ba_print_info("a (setup):\n");
	ba_time_periods_append_start_end(periods_a, 0.0, 1.0);
	
	ba_time_periods_append_start_end(periods_a, 10.0, 11.0);
	ba_time_periods_append_start_end(periods_a, 20.0, 21.0);
	ba_time_periods_append_start_end(periods_a, 30.0, 31.0);
	ba_time_periods_append_start_end(periods_a, 40.0, 41.0);

	ba_time_periods_append_start_end(periods_a, 50.5, 51.0);
	ba_time_periods_append_start_end(periods_a, 60.5, 61.5);
	ba_time_periods_append_start_end(periods_a, 71.0, 72.0);
	ba_time_periods_append_start_end(periods_a, 82.0, 83.0);
	
	ba_time_periods_append_start_end(periods_a, 90.0, 91.0);
	ba_time_periods_append_start_end(periods_a, 95.1, 95.9);

	ba_time_periods_print(periods_a);
	
	ba_print_info("b (setup):\n");
	ba_time_periods_append_start_end(periods_b, 0.0, 1.0);

	ba_time_periods_append_start_end(periods_b, 10.5, 11.0);
	ba_time_periods_append_start_end(periods_b, 20.5, 21.5);
	ba_time_periods_append_start_end(periods_b, 31.0, 32.0);
	ba_time_periods_append_start_end(periods_b, 42.0, 43.0);
	
	ba_time_periods_append_start_end(periods_b, 50.0, 51.0);
	ba_time_periods_append_start_end(periods_b, 60.0, 61.0);
	ba_time_periods_append_start_end(periods_b, 70.0, 71.0);
	ba_time_periods_append_start_end(periods_b, 80.0, 81.0);
	
	ba_time_periods_append_start_end(periods_b, 90.1, 90.9);
	ba_time_periods_append_start_end(periods_b, 95.0, 96.0);

	ba_time_periods_print(periods_b);
	
	ba_print_info("a - b:\n");
	ba_time_periods_difference(periods_a, periods_b, result);
	ba_time_periods_print(result);
	
	ba_print_info("b - a:\n");
	ba_time_periods_difference(periods_b, periods_a, result);
	ba_time_periods_print(result);
	
	ba_print_info("a (re-setup):\n");
	ba_time_periods_nullify(periods_a);
	ba_time_periods_append_start_end(periods_a, 0.0, 1.0);
	ba_time_periods_print(periods_a);
	
	ba_print_info("b (re-setup, this time to null):\n");
	ba_time_periods_nullify(periods_b);
	ba_time_periods_print(periods_b);
	
	ba_print_info("a - b:\n");
	ba_time_periods_difference(periods_a, periods_b, result);
	ba_time_periods_print(result);
	
	ba_print_info("b - a:\n");
	ba_time_periods_difference(periods_b, periods_a, result);
	ba_time_periods_print(result);
	
	ba_time_periods_delete(periods_a);
	ba_time_periods_delete(periods_b);
	ba_time_periods_delete(result);
    }

    {
	ba_time_periods_t *periods_a, *periods_b;
	ba_bool equal;

	ba_print_info("EQUALITY TESTS...........................\n");

	periods_a = ba_time_periods_new();
	periods_b = ba_time_periods_new();

	ba_print_info("a (setup):\n");
	ba_time_periods_append_start_end(periods_a, 0.0, 1.0);
	ba_time_periods_print(periods_a);

	ba_print_info("b (setup):\n");
	ba_time_periods_append_start_end(periods_b, 0.0, 1.0);
	ba_time_periods_append_start_end(periods_b, 1.0, 2.0);
	ba_time_periods_print(periods_b);

	ba_print_info("a == b\n");
	equal = ba_time_periods_is_equal(periods_a, periods_b);
	ba_print_info("answer (should be 0): %d\n", equal);

	ba_print_info("a (append something):\n");
	ba_time_periods_append_start_end(periods_a, 1.0, 2.0);
	ba_time_periods_print(periods_a);

	ba_print_info("a == b\n");
	equal = ba_time_periods_is_equal(periods_a, periods_b);
	ba_print_info("answer (should be 1): %d\n", equal);

	ba_time_periods_delete(periods_a);
	ba_time_periods_delete(periods_b);
    }

    {
	ba_time_periods_t *periods_a;

	ba_print_info("SIMPLIFY TEST...........................\n");

	periods_a = ba_time_periods_new();

	ba_print_info("a (setup: [0,1], [1.1,2], [2,3]):\n");
	ba_time_periods_append_start_end(periods_a, 0.0, 1.0);
	ba_time_periods_append_start_end(periods_a, 1.1, 2.0);
	ba_time_periods_append_start_end(periods_a, 2.0, 3.0);
	ba_time_periods_print(periods_a);

	ba_time_periods_delete(periods_a);
    }

    {
	ba_time_periods_t *periods_a, *periods_b, *result;

	ba_print_info("UNION TEST.........\n");

	periods_a = ba_time_periods_new();
	periods_b = ba_time_periods_new();
	result = ba_time_periods_new();

	ba_print_info("a (setup):\n");
	ba_time_periods_append_start_end(periods_a, 0.0, 1.0);
	
	ba_time_periods_append_start_end(periods_a, 10.0, 11.0);
	ba_time_periods_append_start_end(periods_a, 20.0, 21.0);
	ba_time_periods_append_start_end(periods_a, 30.0, 31.0);
	ba_time_periods_append_start_end(periods_a, 40.0, 41.0);

	ba_time_periods_append_start_end(periods_a, 50.5, 51.0);
	ba_time_periods_append_start_end(periods_a, 60.5, 61.5);
	ba_time_periods_append_start_end(periods_a, 71.0, 72.0);
	ba_time_periods_append_start_end(periods_a, 82.0, 83.0);
	
	ba_time_periods_append_start_end(periods_a, 90.0, 91.0);
	ba_time_periods_append_start_end(periods_a, 95.1, 95.9);

	ba_time_periods_print(periods_a);
	
	ba_print_info("b (setup):\n");
	ba_time_periods_append_start_end(periods_b, 0.0, 1.0);

	ba_time_periods_append_start_end(periods_b, 10.5, 11.0);
	ba_time_periods_append_start_end(periods_b, 20.5, 21.5);
	ba_time_periods_append_start_end(periods_b, 31.0, 32.0);
	ba_time_periods_append_start_end(periods_b, 42.0, 43.0);
	
	ba_time_periods_append_start_end(periods_b, 50.0, 51.0);
	ba_time_periods_append_start_end(periods_b, 60.0, 61.0);
	ba_time_periods_append_start_end(periods_b, 70.0, 71.0);
	ba_time_periods_append_start_end(periods_b, 80.0, 81.0);

	ba_time_periods_append_start_end(periods_b, 90.1, 90.9);
	ba_time_periods_append_start_end(periods_b, 95.0, 96.0);

	ba_time_periods_print(periods_b);
	
	ba_print_info("a u b:\n");
	ba_time_periods_union(periods_a, periods_b, result);
	ba_time_periods_print(result);

	ba_time_periods_delete(periods_a);
	ba_time_periods_delete(periods_b);
	ba_time_periods_delete(result);
    }

    {
	ba_time_periods_t *periods_a, *periods_b, *result;

	ba_print_info("hairy overlapping union / intersection tests......\n");

	periods_a = ba_time_periods_new();
	periods_b = ba_time_periods_new();
	result = ba_time_periods_new();

	ba_print_info("a (setup):\n");
	ba_time_periods_append_start_end(periods_a, 0.0, 10.0);
	ba_time_periods_print(periods_a);

	ba_print_info("b (setup):\n");
	ba_time_periods_append_start_end(periods_b, 1.0, 2.0);
	ba_time_periods_append_start_end(periods_b, 3.0, 5.0);
	ba_time_periods_append_start_end(periods_b, 8.0, 9.0);
	ba_time_periods_print(periods_b);

	ba_print_info("a u b\n");
	ba_time_periods_union(periods_a, periods_b, result);
	ba_time_periods_print(result);

	ba_print_info("a ^ b\n");
	ba_time_periods_intersection(periods_a, periods_b, result);
	ba_time_periods_print(result);

	ba_print_info("b (re-setup):\n");
	ba_time_periods_nullify(periods_b);
	ba_time_periods_append_start_end(periods_b, -1.0, 2.0);
	ba_time_periods_append_start_end(periods_b, 3.0, 5.0);
	ba_time_periods_append_start_end(periods_b, 8.0, 11.0);
	ba_time_periods_print(periods_b);

	ba_print_info("a u b\n");
	ba_time_periods_union(periods_a, periods_b, result);
	ba_time_periods_print(result);

	ba_print_info("a ^ b\n");
	ba_time_periods_intersection(periods_a, periods_b, result);
	ba_time_periods_print(result);

	ba_time_periods_delete(periods_a);
	ba_time_periods_delete(periods_b);
	ba_time_periods_delete(result);
    }

    {
	ba_time_periods_t *set_a, *set_b, *set_c, *result1, *result2, *zero;
	ba_time_period_t *universe;

	set_a = ba_time_periods_new();
	set_b = ba_time_periods_new();
	set_c = ba_time_periods_new();
	result1 = ba_time_periods_new();
	result2 = ba_time_periods_new();
	zero = ba_time_periods_new();

	ba_print_info("AXIOMS-------------------------------------\n");

	ba_print_info("a (setup):\n");
	ba_time_periods_append_start_end(set_a, 0.0, 100.0);
	ba_time_periods_print(set_a);

	ba_print_info("b (setup):\n");
	ba_time_periods_append_start_end(set_b, 50.0, 150.0);
	ba_time_periods_print(set_b);

	ba_print_info("c (setup):\n");
	ba_time_periods_append_start_end(set_c, 75.0, 175.0);
	ba_time_periods_print(set_c);

	ba_print_info("zero (null):\n");
	ba_time_periods_nullify(zero); // not necessary...
	ba_time_periods_print(zero);

	ba_print_info("universe (setup):\n");
	universe = ba_time_period_new(-100.0, 200.0);
	ba_time_period_print(universe);

	{
	    ba_print_info("a ^ 0 = 0\n");
	    ba_time_periods_intersection(set_a, zero, result1);
	    ba_time_periods_print(result1);
	    BA_ASSERT(ba_time_periods_is_equal(result1, zero));
	}

	{
	    ba_print_info("a u 0 = a\n");
	    ba_time_periods_union(set_a, zero, result1);
	    ba_time_periods_print(result1);
	    BA_ASSERT(ba_time_periods_is_equal(result1, set_a));
	}

	{
	    ba_print_info("a ^ a = a\n");
	    ba_time_periods_intersection(set_a, set_a, result1);
	    ba_time_periods_print(result1);
	    BA_ASSERT(ba_time_periods_is_equal(result1, set_a));
	}

	{
	    ba_print_info("a u a = a\n");
	    ba_time_periods_union(set_a, set_a, result1);
	    ba_time_periods_print(result1);
	    BA_ASSERT(ba_time_periods_is_equal(result1, set_a));
	}

	{
	    ba_print_info("a ^ b = b ^ a\n");
	    ba_time_periods_intersection(set_a, set_b, result1);
	    ba_time_periods_intersection(set_b, set_a, result2);
	    ba_time_periods_print(result1);
	    ba_time_periods_print(result2);
	    BA_ASSERT(ba_time_periods_is_equal(result1, result2));
	}

	{
	    ba_time_periods_union(set_a, set_b, result1);
	    ba_time_periods_union(set_b, set_a, result2);
	    ba_print_info("a u b = b u a\n");
	    ba_time_periods_print(result1);
	    ba_time_periods_print(result2);
	    BA_ASSERT(ba_time_periods_is_equal(result1, result2));
	}

	{
	    ba_time_periods_t *tmp1, *tmp2;

	    tmp1 = ba_time_periods_new();
	    tmp2 = ba_time_periods_new();

	    ba_print_info("a ^ (b ^ c) = (a ^ b) ^ c\n");
	    ba_time_periods_intersection(set_b, set_c, tmp1);
	    ba_time_periods_intersection(set_a, tmp1, result1);

	    ba_time_periods_intersection(set_a, set_b, tmp2);
	    ba_time_periods_intersection(tmp2, set_c, result2);

	    ba_time_periods_print(result1);
	    ba_time_periods_print(result2);
	    BA_ASSERT(ba_time_periods_is_equal(result1, result2));

	    ba_time_periods_delete(tmp1);
	    ba_time_periods_delete(tmp2);
	}

	{
	    ba_time_periods_t *tmp1, *tmp2;

	    tmp1 = ba_time_periods_new();
	    tmp2 = ba_time_periods_new();

	    ba_print_info("a u (b u c) = (a u b) u c\n");
	    ba_time_periods_union(set_b, set_c, tmp1);
	    ba_time_periods_union(set_a, tmp1, result1);

	    ba_time_periods_union(set_a, set_b, tmp2);
	    ba_time_periods_union(tmp2, set_c, result2);

	    ba_time_periods_print(result1);
	    ba_time_periods_print(result2);
	    BA_ASSERT(ba_time_periods_is_equal(result1, result2));

	    ba_time_periods_delete(tmp1);
	    ba_time_periods_delete(tmp2);
	}

	{
	    ba_time_periods_t *tmp1, *tmp2, *tmp3;

	    tmp1 = ba_time_periods_new();
	    tmp2 = ba_time_periods_new();
	    tmp3 = ba_time_periods_new();

	    ba_print_info("a ^ (b u c) = (a ^ b) u (a ^ c)\n");
	    ba_time_periods_union(set_b, set_c, tmp1);
	    ba_time_periods_intersection(set_a, tmp1, result1);

	    ba_time_periods_intersection(set_a, set_b, tmp2);
	    ba_time_periods_intersection(set_a, set_c, tmp3);
	    ba_time_periods_union(tmp2, tmp3, result2);

	    ba_time_periods_print(result1);
	    ba_time_periods_print(result2);
	    BA_ASSERT(ba_time_periods_is_equal(result1, result2));

	    ba_time_periods_delete(tmp1);
	    ba_time_periods_delete(tmp2);
	    ba_time_periods_delete(tmp3);
	}

	{
	    ba_time_periods_t *tmp1, *tmp2, *tmp3;

	    tmp1 = ba_time_periods_new();
	    tmp2 = ba_time_periods_new();
	    tmp3 = ba_time_periods_new();

	    ba_print_info("a u (b ^ c) = (a u b) ^ (a u c)\n");
	    ba_time_periods_intersection(set_b, set_c, tmp1);
	    ba_time_periods_union(set_a, tmp1, result1);

	    ba_time_periods_union(set_a, set_b, tmp2);
	    ba_time_periods_union(set_a, set_c, tmp3);
	    ba_time_periods_intersection(tmp2, tmp3, result2);

	    ba_time_periods_print(result1);
	    ba_time_periods_print(result2);
	    BA_ASSERT(ba_time_periods_is_equal(result1, result2));

	    ba_time_periods_delete(tmp1);
	    ba_time_periods_delete(tmp2);
	    ba_time_periods_delete(tmp3);
	}

	{
	    ba_time_periods_t *tmp;

	    tmp = ba_time_periods_new();

	    ba_print_info("a ^ (a u b) = a\n");
	    ba_time_periods_union(set_a, set_b, tmp);
	    ba_time_periods_intersection(set_a, tmp, result1);

	    ba_time_periods_print(result1);
	    ba_time_periods_print(set_a);
	    BA_ASSERT(ba_time_periods_is_equal(result1, set_a));

	    ba_time_periods_delete(tmp);
	}

	{
	    ba_time_periods_t *tmp;

	    tmp = ba_time_periods_new();

	    ba_print_info("a u (a ^ b) = a\n");
	    ba_time_periods_intersection(set_a, set_b, tmp);
	    ba_time_periods_union(set_a, tmp, result1);

	    ba_time_periods_print(result1);
	    ba_time_periods_print(set_a);
	    BA_ASSERT(ba_time_periods_is_equal(result1, set_a));

	    ba_time_periods_delete(tmp);
	}

	{
	    ba_time_periods_t *tmp1, *tmp2, *tmp3;

	    tmp1 = ba_time_periods_new();
	    tmp2 = ba_time_periods_new();
	    tmp3 = ba_time_periods_new();

	    ba_print_info("a - (b ^ c) = (a - b) u (a - c)\n");
	    ba_time_periods_intersection(set_b, set_c, tmp1);
	    ba_time_periods_difference(set_a, tmp1, result1);

	    ba_time_periods_difference(set_a, set_b, tmp2);
	    ba_time_periods_difference(set_a, set_c, tmp3);
	    ba_time_periods_union(tmp2, tmp3, result2);

	    ba_time_periods_print(result1);
	    ba_time_periods_print(result2);
	    BA_ASSERT(ba_time_periods_is_equal(result1, result2));

	    ba_time_periods_delete(tmp1);
	    ba_time_periods_delete(tmp2);
	    ba_time_periods_delete(tmp3);
	}

	{
	    ba_time_periods_t *tmp1, *tmp2, *tmp3;

	    tmp1 = ba_time_periods_new();
	    tmp2 = ba_time_periods_new();
	    tmp3 = ba_time_periods_new();

	    ba_print_info("a - (b u c) = (a - b) ^ (a - c)\n");
	    ba_time_periods_union(set_b, set_c, tmp1);
	    ba_time_periods_difference(set_a, tmp1, result1);

	    ba_time_periods_difference(set_a, set_b, tmp2);
	    ba_time_periods_difference(set_a, set_c, tmp3);
	    ba_time_periods_intersection(tmp2, tmp3, result2);

	    ba_time_periods_print(result1);
	    ba_time_periods_print(result2);
	    BA_ASSERT(ba_time_periods_is_equal(result1, result2));

	    ba_time_periods_delete(tmp1);
	    ba_time_periods_delete(tmp2);
	    ba_time_periods_delete(tmp3);
	}

	{
	    ba_time_periods_t *tmp;

	    tmp = ba_time_periods_new();

	    ba_print_info("not not a = a\n");
	    ba_time_periods_complement(set_a, universe, tmp);
	    ba_time_periods_complement(tmp, universe, result1);

	    ba_time_periods_print(result1);
	    ba_time_periods_print(set_a);
	    BA_ASSERT(ba_time_periods_is_equal(result1, set_a));

	    ba_time_periods_delete(tmp);
	}

	{
	    ba_time_periods_t *tmp;

	    tmp = ba_time_periods_new();

	    ba_print_info("a ^ not a = 0\n");
	    ba_time_periods_complement(set_a, universe, tmp);
	    ba_time_periods_intersection(set_a, tmp, result1);

	    ba_time_periods_print(result1);
	    ba_time_periods_print(zero);
	    BA_ASSERT(ba_time_periods_is_equal(result1, zero));

	    ba_time_periods_delete(tmp);
	}

	{
	    ba_time_periods_t *tmp;
	    
	    tmp = ba_time_periods_new();

	    ba_print_info("a u not a = U\n");
	    ba_time_periods_complement(set_a, universe, tmp);
	    ba_time_periods_union(set_a, tmp, result1);
	    ba_time_periods_print(result1);
	    BA_ASSERT(ba_time_periods_is_equal_period(result1, universe));

	    ba_time_periods_delete(tmp);
	}

	ba_time_periods_delete(set_a);
	ba_time_periods_delete(set_b);
	ba_time_periods_delete(set_c);
	ba_time_periods_delete(result1);
	ba_time_periods_delete(result2);
	ba_time_periods_delete(zero);
	ba_time_period_delete(universe);
    }
}
